import { TypedEmitter } from "tiny-typed-emitter";
import sql from "better-sqlite3";
import { DatabaseEvents, DatabaseOptions, DataResolvable, SortOptions, SqliteChanges, SqliteQueryOptions, TableResolvable } from "../typings";
import { Table } from "./Table";
import { QuerySelector } from "../builders/QuerySelector";
/**
 * Database wrapper class for sqlite.
 */
declare class Database<T = unknown> extends TypedEmitter<DatabaseEvents> {
    #private;
    /**
     * The sqlite database.
     */
    db?: sql.Database;
    /**
     * Date at which the database started.
     */
    readyAt?: Date;
    /**
     * The options for this database.
     */
    options: DatabaseOptions;
    /**
     * The assigned tables for this database.
     */
    tables: Map<string, Table>;
    /**
     *
     * @param options The options for this database.
     */
    constructor(options?: DatabaseOptions);
    get query(): QuerySelector<T>;
    typedGet<K extends keyof T>(table: K, options: string | SqliteQueryOptions): T[K];
    /**
     * Validates given options.
     * @param options The options to check.
     * @returns The options with their default values.
     */
    private _validateOptions;
    deleteTimedData(table: string, key: string): boolean;
    setTimedData(table: string, key: string, duration: number): this;
    hasTimeoutsTable(): boolean;
    /**
     * Resolves a string or object into a table.
     * @param name The name of the table.
     * @param suppress Whether to suppress the error if table does not exist.
     * @returns The table found.
     */
    resolveTable(name: TableResolvable, suppress?: boolean): Table;
    /**
     * Deletes garbage from this database.
     * @returns
     */
    vacuum(): any;
    /**
     * Creates a table.
     * @param name The name for this table
     * @returns The table that was created.
     */
    createTable(name: string): Table;
    /**
     * Adds a table to this database.
     * @param table The table to add.
     * @returns The database this table was added to.
     */
    addTable(table: Table): this;
    /**
     * Returns the corresponding array from given map.
     * @param map The map to convert.
     * @returns The array from the map.
     */
    arrayFromMap(map: any): Table[];
    get readyTimestamp(): number | undefined;
    /**
     * Gets data from a table.
     * @param name The name of the table to get its data.
     * @returns The table data if it exists.
     */
    getTableData(name: string): Record<string, any> | undefined;
    /**
     * Gets all available tables from this database.
     * @returns An array of table data.
     */
    get databaseTables(): Record<string, any>[] | undefined;
    /**
     * Gets multiple rows from this database.
     * @param table The table to get rows from.
     * @param options The options to apply for this query.
     * @returns The rows found from this query.
     */
    all<K = undefined>(table: TableResolvable, options?: SqliteQueryOptions): K extends undefined ? DataResolvable[] : K[];
    /**
     * Gets the position of a row in a table.
     * @param table The table to get row position from.
     * @param sortOptions Options for sorting this query.
     * @param whereOptions Options to filter this query.
     * @returns The position or positions of rows (and possibly their data if multiple).
     * @example
     * const posOf = db.get("mytable", {
     *      columns: ["bank"]
     * }, {
     *      where: {
     *          column: "id",
     *          equals: "123456"
     *      }
     * })
     *
     * console.log(posOf.position)
     */
    getRowPosition(table: TableResolvable, sortOptions: SortOptions, whereOptions?: SqliteQueryOptions): {
        position: number;
    } | DataResolvable[];
    /**
     * Get total row count in a table.
     * @param table The table to get row count from.
     * @param options The options for this query.
     * @returns An object with the count property.
     */
    getRowCount(table: TableResolvable, options?: SqliteQueryOptions): {
        count: number;
    };
    /**
     * Sets a row into the database (or updates existing one).
     * @param table The table to perform this action on.
     * @param data The data to set.
     * @param options Options for this query.
     * @returns Sqlite result for this setter.
     */
    set<K extends keyof T>(table: K, data: T[K]): (SqliteChanges | undefined)[];
    set<K extends keyof T>(table: K, data: T[K], options: SqliteQueryOptions): (SqliteChanges | undefined)[];
    set<K = undefined>(table: TableResolvable, data: K extends undefined ? DataResolvable : K): (SqliteChanges | undefined)[];
    set<K = undefined>(table: TableResolvable, data: K extends undefined ? DataResolvable : K, options?: SqliteQueryOptions): (SqliteChanges | undefined)[];
    /**
     * Deletes rows from a database, if no options are given it will clean the entire table.
     * @param table The table to delete rows from.
     * @param options Options for this query.
     * @returns Affected rows and changes from the sqlite table.
     */
    delete<K extends keyof T>(table: K, key: string): SqliteChanges;
    delete<K extends keyof T>(table: K, options: SqliteQueryOptions): SqliteChanges;
    delete(table: TableResolvable, primaryKey: string): SqliteChanges;
    delete(table: TableResolvable, options?: SqliteQueryOptions): SqliteChanges;
    /**
     * Insert one or multiple rows into the database.
     * If a row with given primary key exists, this will throw an error.
     * @param table The table to set these rows to.
     * @param data The row or rows to insert.
     * @returns The sqlite changes happened in the table.
     */
    insert<K extends keyof T>(table: K, data: T[K] | (T[K])[]): ReturnType<Table["insert"]>;
    insert<K = undefined>(table: string, data: K extends undefined ? DataResolvable[] | DataResolvable : K): SqliteChanges;
    /**
     * Gets a single row from the database.
     * @param table The table to get a row from.
     * @param options The options for this query.
     * @returns A row from the database, if it exists.
     * @example
     * db.get("mytable", {
     *      where: {
     *          column: "id",
     *          equals: "12345"
     *      }
     * })
     */
    get<K extends keyof T>(table: K, options: SqliteQueryOptions): T[K];
    get<K extends keyof T>(table: K, key: string): T[K];
    get<K = undefined>(table: TableResolvable, primaryKey: string): K extends undefined ? DataResolvable : K;
    get<K = undefined>(table: TableResolvable, options: SqliteQueryOptions): K extends undefined ? DataResolvable : K;
    addTimeoutsTable(): this;
    /**
     * Connects the database to its corresponding file.
     * @returns The connected database.
     */
    connect(): Promise<this>;
    /**
     * Closes this database connection.
     * @returns
     */
    close(): true;
}
export default Database;
//# sourceMappingURL=Database.d.ts.map